%==================================================================
%  M-file's name:     Parameter.m
%  This file initializes all the parameters required for simulation
%  Run this file before running the simulation OR
%  Run this file after envoking the simulink file 
%  but before running the simulation.
%
%  This program generates the reduced order model in the modal
%  coordinates for simulation purpose.
%
%                                    Written by Rajesh Adhikari
%                                    Saitama University, Japan
%
%  The following parameters must be set before running this program
%  Tf    = 10.0;      % For El Centro earthquake
%  Tf    = 7.2;       % For Hachinohe
%  Tf    = 300.0;     % For Filtered Random Excitation input
%  Xg_RMS= 0.1657;    % For El Centro earthquake
%  Xg_RMS= 0.1408;    % For Hachinohe earthquake
%  Xg_RMS= 0.1;       % Arbitrary for Filtered Random Excitation input
%==================================================================
   Tf    = 10.0;
   %eaj Xg_RMS= 0.1657;

%=== Load the Structural Model (Evaluation Model)
load eval_mod;         % Matrices stored in   eval_mod   are as 
                       % (A,B,E,Cy,Dy,Fy,Cz,Dz,Fz)

% FORM KANAI-TAJIMI FILTER 
% =========================
%eaj wg  = 50;                     % You may want to change this value
%eaj zg  = 0.3;                    % You may want to change this value
S0 	= 0.03*zg/(pi*wg*(4*zg^2+1));
numeq	= sqrt(S0)*[2*zg*wg wg^2];
deneq	= [1 2*zg*wg wg^2];

% SET UP STATE-SPACE SYSTEM FOR PLANT
% ===================================
%     INPUTS:      
%          1) Ground Acceleration
%          2) Control Signal (Actuator Command
%     OUTPUTS:	
%          1-12) Z vector (see paper)
%          13) Ground Acceleration (directly fed through)
%          xy  The response in the modal coordinates
%          sxsy The sliding surface sigma(x,t)

[At] = [A];
[Bt] = [E B];
[Ct] = [Cz;zeros(1,length(At))];
[Dt] = [[Fz Dz]; [1 0]];

% Set Paramater for Simulation
% ===========================
dtint = 0.0001;     % Integration step
dts   = 0.005;      % Time increment for saving data
%Tf    = 10.0;       % Maximum time for El Centro Earthquake
                    % Set Tf = 7.2 for Hachinohe
n     = 1e7;        % Number of samples saved
tol   = 1e-6;       % Tolerance
qint  = 6/(2^12);   % Quantizer interval
%
% === Complex Eigenvalue Analysis --> Modal Synthesis
% ===================================================
%
[PHI_R,T] =  eig(A);                  % Right Eigenvectors
PHI_L     = (inv(PHI_R))';            % Left Eigenvector

%==== Arrange the Eigenvecots in complex and its conjugate parts
%                  1st Modal vector PHI_R(:,25:26)
%                  2nd Modal vector PHI_R(:,22:23)
%                  3rd Modal vector PHI_R(:,20:21)

PHI_R11 = [ PHI_R(:,25) PHI_R(:,22) PHI_R(:,20) PHI_R(:,1) PHI_R(:,2)];
PHI_R12 = [ PHI_R(:,3)  PHI_R(:,5)  PHI_R(:,7)  PHI_R(:,9) PHI_R(:,11)];
PHI_R13 = [ PHI_R(:,13) PHI_R(:,15) PHI_R(:,16) PHI_R(:,18) PHI_R(:,24) PHI_R(:,27)];

PHI_R21 = [ PHI_R(:,26) PHI_R(:,23) PHI_R(:,21) PHI_R(:,1) PHI_R(:,2)];
PHI_R22 = [ PHI_R(:,4)  PHI_R(:,6)  PHI_R(:,8)  PHI_R(:,10) PHI_R(:,12)];
PHI_R23 = [ PHI_R(:,14) PHI_R(:,15) PHI_R(:,17) PHI_R(:,19) PHI_R(:,24) PHI_R(:,28)];

PHI_L11 = [ PHI_L(:,25) PHI_L(:,22) PHI_L(:,20) PHI_L(:,1) PHI_L(:,2)];
PHI_L12 = [ PHI_L(:,3)  PHI_L(:,5)  PHI_L(:,7)  PHI_L(:,9) PHI_L(:,11)];
PHI_L13 = [ PHI_L(:,13) PHI_L(:,15) PHI_L(:,16) PHI_L(:,18) PHI_L(:,24) PHI_L(:,27)];

PHI_L21 = [ PHI_L(:,26) PHI_L(:,23) PHI_L(:,21) PHI_L(:,1) PHI_L(:,2)];
PHI_L22 = [ PHI_L(:,4)  PHI_L(:,6)  PHI_L(:,8)  PHI_L(:,10) PHI_L(:,12)];
PHI_L23 = [ PHI_L(:,14) PHI_L(:,15) PHI_L(:,17) PHI_L(:,19) PHI_L(:,24) PHI_L(:,28)];

PHI_R = [PHI_R11 PHI_R12 PHI_R13  PHI_R21 PHI_R22 PHI_R23];
PHI_L = [PHI_L11 PHI_L12 PHI_L13  PHI_L21 PHI_L22 PHI_L23];

%==== Check the Biorthonormal properties of the eigenvectors
%diag(PHI_L'*PHI_R)
%diag(PHI_L'*A*PHI_R)
%diag(T)


% How Many modes to be controlled ? -- We use only one mode
% =========================================================
N_Modes = 1;
PHI_LC   = PHI_L(:,1:N_Modes);       % Left  eigenvectors for Controlled modes
PHI_RC   = PHI_R(:,1:N_Modes);       % Right eigenvectors for Controlled modes
PHI_LN   = PHI_L(:,N_Modes+1:16);    % Left  eigenvectors for Unontrolled modes
PHI_RN   = PHI_R(:,N_Modes+1:16);    % Right eigenvectors for Unontrolled modes

% System matrices for the Controlled modes
%=========================================
A_c11  = PHI_LC'*A*PHI_RC;
A_c    = [ real(A_c11)  -imag(A_c11);
           imag(A_c11)   real(A_c11)];
B_c    = [ real(PHI_LC'*B);
           imag(PHI_LC'*B)];
E_c    = [ real(PHI_LC'*E);
           imag(PHI_LC'*E)];
        
% ================================================================
% Feedback responses are the Acceleration, and AMD's displacement
% ================================================================
Cyr_c    = 2*(Cy(4:5,:))*([real(PHI_RC)  -1.0*imag(PHI_RC)]);
Cyr_n    = 2*(Cy(4:5,:))*([real(PHI_RN)  -1.0*imag(PHI_RN)]);
Fyr_c    =  Fy(4:5,:);

% DESIGN THE Kalman filter Observer gain  
% =====================================
SW = 25;                                 %  sensor noise levels (see the paper)
SV = eye(size(Cyr_c,1));                             % 
L  = lqew(A_c,E_c,Cyr_c,Fyr_c,SW,SV);    % Kalman filter gain


% Controller Design
% ================
% design the Sliding Surface Coefficient Matrix (S)
%--------------------------------------------------
Q  = [10 0  
       0 1];
A11 = A_c(1,1);
A12 = A_c(1,2);
Q11 = Q(1,1);
Q22 = Q(2,2);
[KK,EE] = lqr(A11,A12,Q11,Q22);
S = [A12'*EE,Q22];

u_eq =  -(inv(S*B_c))*(S*A_c);              % Equivalent Control gai
bw   = 0.1;                                 % Boundary layer width  
RHO  = abs(inv(S*B_c)*(S*L*Fyr_c*Xg_RMS));  % From Xg_RMS (Elcentro = 0.1657)
                                            % For Hachinohe, replace it by the
                                            % Xg_RMS for Hachinohe
disp(' ')   
disp('THE CURRENT SELECTED VALUES ARE:')
disp('================================ ')
TT=[S RHO u_eq bw];                             % To Print Data on the Screen
fprintf(1,'          S                 Rho             Ueq              epsilon\n')
fprintf(1,'   ---------------       ========  ====================      =======  \n')
fprintf(1,' %8.3f %8.3f     %8.3f  %8.3f      %8.3f         %g\n',TT')
clear TT

% Obtain  Discrete  Controller     
% ============================
MAT_A     = [A_c-(L*Cyr_c)];
MAT_B     = [L   (B_c-(L*Dy(4:5,:)))];
CCc       = eye(2);         
DDc       = zeros(length(CCc),3);

[Adr,Bdr,Cdr,Ddr]=c2dm(MAT_A,MAT_B,CCc,DDc,0.001,'tustin');
